# pattern 自定义正则校验

### 概述

- 简单的长度校验推荐使用 <a href="http://groups.alidemo.cn/hive/form-render/#/docs/prop-schema?type=html" target="_blank">propSchema</a> 中的属性
- pattern 是存在于 propSchema 中属性的一个字段，长用在当长度限制还不满足要求的场景，可通过正则的方式来对用户输入项进行校验
- pattern 和 required、minLength、maxLength 等这一类显示为`&&`的关系，当有一个不满足时候，onValidate 回调中将会返回不符合的数组项

### 格式

- 格式和 JS 正则的格式保持一致，实际为此正则类中的 pattern 字段 `new RegExp(pattern).test(value)`

### 常见 pattern 规则：

| 字符             | 含意                                                                                                                                                                                                                                                                                                                                                                                                                      |
| ---------------- | ------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- |
| \                | 将下一个字符标记为一个特殊字符、或一个原义字符、或一个后向引用、或一个八进制转义符。 例如，’n’ 匹配字符 “n”。’\n’ 匹配一个换行符。序列 ‘' 匹配 “” 而 “\(” 则匹配 “(“。                                                                                                                                                                                                                                                    |
| ^                | 匹配输入字符串的开始位置。如果设置了 RegExp 对象的 Multiline 属性，^ 也匹配 ‘\n’ 或 ‘\r’ 之后的位置。                                                                                                                                                                                                                                                                                                                     |
| \$               | 匹配输入字符串的结束位置。如果设置了 RegExp 对象的 Multiline 属性，\$ 也匹配 ‘\n’ 或 ‘\r’ 之前的位置。                                                                                                                                                                                                                                                                                                                    |
| \*               | 匹配前面的子表达式零次或多次。例如，zo* 能匹配 “z” 以及 “zoo”。 * 等价于{0,}。                                                                                                                                                                                                                                                                                                                                            |
| +                | 匹配前面的子表达式一次或多次。例如，’zo+’ 能匹配 “zo” 以及 “zoo”，但不能匹配 “z”。+ 等价于 {1,}。                                                                                                                                                                                                                                                                                                                         |
| ?                | 匹配前面的子表达式零次或一次。例如，”do(es)?” 可以匹配 “do” 或 “does” 中的”do” 。? 等价于 {0,1}。                                                                                                                                                                                                                                                                                                                         |
| {n}              | n 是一个非负整数。匹配确定的 n 次。例如，’o{2}’ 不能匹配 “Bob” 中的 ‘o’，但是能匹配 “food” 中的两个 o。                                                                                                                                                                                                                                                                                                                   |
| {n,}             | n 是一个非负整数。至少匹配 n 次。 例如: ‘o{2,}’ 不能匹配 “Bob” 中的 ‘o’，但能匹配 “foooood” 中的所有 o。 ‘o{1,}’ 等价于 ‘o+’。’o{0,}’ 则等价于 ‘o\*’。                                                                                                                                                                                                                                                                    |
| {n,m}            | m 和 n 均为非负整数，其中 n <= m。最少匹配 n 次且最多匹配 m 次。例如: “o{1,3}” 将匹配 “fooooood” 中的前三个 o。’o{0,1}’ 等价于 ‘o?’。请注意在逗号和两个数之间不能有空格。                                                                                                                                                                                                                                                 |
| ?                | 当该字符紧跟在任何一个其他限制符 (\*, +, ?, {n}, {n,}, {n,m}) 后面时，匹配模式是非贪婪的。 非贪婪模式尽可能少的匹配所搜索的字符串，而默认的贪婪模式则尽可能多的匹配所搜索的字符串。 例如，对于字符串 “oooo”，’o+?’ 将匹配单个 “o”，而 ‘o+’ 将匹配所有 ‘o’。                                                                                                                                                               |
| .                | 匹配除 “\n” 之外的任何单个字符。要匹配包括 ‘\n’ 在内的任何字符，请使用象 ‘[.\n]’ 的模式。                                                                                                                                                                                                                                                                                                                                 |
| (pattern)        | 匹配 pattern 并获取这一匹配。 所获取的匹配可以从产生的 Matches 集合得到，要匹配圆括号字符，请使用 ‘‘或‘‘或‘’。                                                                                                                                                                                                                                                                                                            |
| (?:pattern)      | 匹配 pattern 但不获取匹配结果，也就是说这是一个非获取匹配，不进行存储供以后使用。 这在使用 “或” 字符 (\|) 来组合一个模式的各个部分是很有用。 例如， ‘industr(?:y\|ies) 就是一个比 ‘industry\|industries’ 更简略的表达式。                                                                                                                                                                                                 |
| (?=pattern)      | **肯定** 顺序环视 **：子表达式能够匹配右侧的文本**正向预查，在任何匹配 pattern 的字符串开始处匹配查找字符串。这是一个非获取匹配，也就是说，该匹配不需要获取供以后使用。例如， ‘Windows (?=95\|98\|NT\|2000)’ 能匹配 “Windows 2000” 中的 “Windows” ， 但不能匹配 “Windows 3.1” 中的 “Windows”。_预查不消耗字符，也就是说，在一个匹配发生后，在最后一次匹配之后立即开始下一次匹配的搜索， 而不是从包含预查的字符之后开始。_ |
| (?!pattern)      | **否定顺序环视：子表达式不能匹配右侧的文本**负向预查，在任何不匹配的字符串开始处匹配查找字符串。这是一个非获取匹配，也就是说，该匹配不需要获取供以后使用。例如’Windows (?!95\|98\|NT\|2000)’ 能匹配 “Windows 3.1” 中的 “Windows”， 但不能匹配 “Windows 2000” 中的 “Windows”。_预查不消耗字符，也就是说，在一个匹配发生后，在最后一次匹配之后立即开始下一次匹配的搜索， 而不是从包含预查的字符之后开始_                    |
| (?<=pattern))))) | **肯定逆序环视：子表达式能够匹配左侧的文本**                                                                                                                                                                                                                                                                                                                                                                              |
| (?<!pattern))))) | **否定逆序环视：子表达式不能匹配左侧的文本**                                                                                                                                                                                                                                                                                                                                                                              |
| x\|y             | 匹配 x 或 y。例如，’z\|food’ 能匹配 “z” 或 “food”。’(z\|f)ood’ 则匹配 “zood” 或 “food”。                                                                                                                                                                                                                                                                                                                                  |
| [xyz]            | 字符集合。匹配所包含的任意一个字符。例如， ‘[abc]’ 可以匹配 “plain” 中的 ‘a’。                                                                                                                                                                                                                                                                                                                                            |
| [^xyz]           | 负值字符集合。匹配未包含的任意字符。例如， ‘[^abc]’ 可以匹配 “plain” 中的’p’。                                                                                                                                                                                                                                                                                                                                            |
| [a-z]            | 字符范围。匹配指定范围内的任意字符。例如，’[a-z]’ 可以匹配 ‘a’ 到 ‘z’ 范围内的任意小写字母字符。                                                                                                                                                                                                                                                                                                                          |
| [^a-z]           | 负值字符范围。匹配任何不在指定范围内的任意字符。 例如，’[^a-z]’ 可以匹配任何不在 ‘a’ 到 ‘z’ 范围内的任意字符。                                                                                                                                                                                                                                                                                                            |
| \b               | 匹配一个单词边界，也就是指单词和空格间的位置。 例如， ‘er\b’ 可以匹配”never” 中的 ‘er’，但不能匹配 “verb” 中的 ‘er’。                                                                                                                                                                                                                                                                                                     |
| \B               | 匹配非单词边界。’er\B’ 能匹配 “verb” 中的 ‘er’，但不能匹配 “never” 中的 ‘er’。                                                                                                                                                                                                                                                                                                                                            |
| \cx              | 匹配由 x 指明的控制字符。 例如， \cM 匹配一个 Control-M 或回车符。 x 的值必须为 A-Z 或 a-z 之一。否则将 c 视为一个原义的’c’字符。                                                                                                                                                                                                                                                                                         |
| \d               | 匹配一个数字字符。等价于 [0-9]。                                                                                                                                                                                                                                                                                                                                                                                          |
| \D               | 匹配一个非数字字符。等价于 [^0-9]。                                                                                                                                                                                                                                                                                                                                                                                       |
