'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.DefaultFieldUI = exports.asField = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _validate = require('./validate');

var _isHidden = require('./isHidden');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

// 解析字符串值
var parseString = function parseString(string, value, rootValue, formData) {
  return Function('"use strict";\n    const value = ' + JSON.stringify(value) + ';\n    const rootValue = ' + JSON.stringify(rootValue) + ';\n    const formData = ' + JSON.stringify(formData) + ';\n    return (' + string + ')')();
};

// asField拆分成逻辑组件和展示组件，从而可替换展示组件的方式完全插拔fr的样式
var asField = function asField(_ref) {
  var FieldUI = _ref.FieldUI,
      Widget = _ref.Widget;

  var FieldContainer = function FieldContainer(_ref2) {
    var className = _ref2.className,
        column = _ref2.column,
        showValidate = _ref2.showValidate,
        isRoot = _ref2.isRoot,
        hidden = _ref2.hidden,
        props = _ref2.props,
        showDescIcon = _ref2.showDescIcon,
        width = _ref2.width,
        disabled = _ref2.disabled,
        readonly = _ref2.readonly,
        options = _ref2.options,
        schema = _ref2.schema,
        rest = _objectWithoutProperties(_ref2, ['className', 'column', 'showValidate', 'isRoot', 'hidden', 'props', 'showDescIcon', 'width', 'disabled', 'readonly', 'options', 'schema']);

    var displayType = rest.displayType,
        value = rest.value,
        _rest$rootValue = rest.rootValue,
        rootValue = _rest$rootValue === undefined ? {} : _rest$rootValue,
        _rest$formData = rest.formData,
        formData = _rest$formData === undefined ? {} : _rest$formData,
        dependShow = rest.dependShow;
    // every key of schema, disabled, readonly, options, hidden, support for function expression

    var convertValue = function convertValue(item) {
      if (typeof item === 'function') {
        return item(value, rootValue, formData);
      } else if (typeof item === 'string' && item.substring(0, 1) === '@') {
        var _item = item.substring(1);
        try {
          return parseString(_item, value, rootValue, formData);
        } catch (error) {
          console.error(error.message);
          console.error('happen at ' + item);
          return item;
        }
      }
      return item;
    };

    hidden = convertValue(hidden);
    disabled = convertValue(disabled);
    readonly = convertValue(readonly);
    options = convertValue(options);
    // iterate over schema, and convert every key
    var _schema = _extends({}, schema);
    Object.keys(schema).forEach(function (key) {
      _schema[key] = convertValue(schema[key]);
    });

    // "ui:hidden": true, hide formItem
    // after "convertValue" being stable, this api will be discarded
    if (hidden && (0, _isHidden.isHidden)({ hidden: hidden, rootValue: rootValue, formData: formData })) {
      return null;
    }

    // 传入组件的值
    var _rest = _extends({}, rest, {
      schema: _schema,
      disabled: disabled,
      readonly: readonly,
      options: options,
      formData: formData || {},
      rootValue: rootValue || {}
    });

    // 不建议使用ui:dependShow, 一般一律使用ui:hidden。ui:dependShow可以做复杂、跨结构的校验
    if ((0, _isHidden.isDependShow)({ formData: formData, dependShow: dependShow })) {
      return null;
    }
    var isComplex = _schema.type === 'object' || _schema.type === 'array' && _schema.enum === undefined;

    var validateText = (0, _validate.getValidateText)(_rest);

    // 必填*，label，描述，竖排时的校验语，只要存在一个，label就不为空
    var showLabel = _schema.title || rest.description || rest.required || displayType !== 'row' && showValidate && validateText;

    var columnStyle = {};
    if (!isComplex && width) {
      columnStyle = {
        width: width,
        paddingRight: '24px'
      };
    } else if (!isComplex && column > 1) {
      columnStyle = {
        width: 'calc(100% /' + column + ')',
        paddingRight: '24px'
      };
    }

    var fieldProps = {
      className: className,
      columnStyle: columnStyle,
      displayType: displayType,
      isComplex: isComplex,
      isRequired: rest.required,
      isRoot: isRoot,
      schema: _schema,
      showDescIcon: showDescIcon,
      showLabel: showLabel,
      showValidate: showValidate,
      validateText: validateText
    };
    return _react2.default.createElement(
      FieldUI,
      fieldProps,
      _react2.default.createElement(Widget, _rest)
    );
  };
  FieldContainer.propTypes = {
    showValidate: _propTypes2.default.bool,
    column: _propTypes2.default.number,
    isRoot: _propTypes2.default.bool,
    props: _propTypes2.default.object,
    showDescIcon: _propTypes2.default.bool,
    displayType: _propTypes2.default.string
  };

  FieldContainer.defaultProps = {
    showValidate: true,
    column: 1,
    isRoot: false,
    props: {},
    showDescIcon: false,
    displayType: 'column'
  };

  return FieldContainer;
};

exports.asField = asField;
var DefaultFieldUI = exports.DefaultFieldUI = function DefaultFieldUI(_ref3) {
  var children = _ref3.children,
      className = _ref3.className,
      columnStyle = _ref3.columnStyle,
      displayType = _ref3.displayType,
      isComplex = _ref3.isComplex,
      isRequired = _ref3.isRequired,
      isRoot = _ref3.isRoot,
      schema = _ref3.schema,
      showDescIcon = _ref3.showDescIcon,
      showLabel = _ref3.showLabel,
      showValidate = _ref3.showValidate,
      validateText = _ref3.validateText;
  var title = schema.title;
  // field 整体 label 标签 content 内容

  var type = schema.type,
      _enum = schema.enum,
      _schema$description = schema.description,
      description = _schema$description === undefined ? '' : _schema$description;

  var fieldClass = 'fr-field w-100 ' + (isComplex ? 'fr-field-complex' : '');
  var labelClass = 'fr-label flex mb2';
  var contentClass = 'fr-content';

  switch (type) {
    case 'object':
      if (title) {
        labelClass += ' fr-label-object bb b--black-20 pb2 mt2 mb3'; // fr-label-object 无默认style，只是占位用于使用者样式覆盖
      }
      if (!isRoot) {
        fieldClass += ' fr-field-object'; // object的margin bottom由内部元素撑起
        if (title) {
          contentClass += ' ml3'; // 缩进
        }
      }
      break;
    case 'array':
      if (title && !_enum) {
        labelClass += ' mt2 mb3';
      }
      break;
    case 'boolean':
      if (title) {
        labelClass += ' ml2';
        if (displayType === 'row') {
          labelClass = labelClass.replace('mb2', 'mb0');
        }
      }
      fieldClass += ' flex flex-row-reverse justify-end';
      break;
    default:
      if (displayType === 'row') {
        labelClass = labelClass.replace('mb2', 'mb0');
      }
  }
  // 横排时
  if (displayType === 'row' && !isComplex && type !== 'boolean') {
    fieldClass += ' flex items-center';
    labelClass += ' flex-shrink-0 fr-label-row mr2';
    labelClass = labelClass.replace('mb2', 'mb0');
    contentClass += ' flex-grow-1 relative';
  }
  return _react2.default.createElement(
    'div',
    {
      className: className ? className + ' ' + fieldClass : fieldClass,
      style: columnStyle
    },
    showLabel && _react2.default.createElement(
      'label',
      { className: labelClass, title: title },
      isRequired && _react2.default.createElement(
        'span',
        { className: 'fr-label-required' },
        ' *'
      ),
      _react2.default.createElement(
        'span',
        { className: isComplex ? 'b' : '' },
        title
      ),
      description && (showDescIcon ? _react2.default.createElement(
        'span',
        { className: 'fr-tooltip-toggle', 'aria-label': description },
        _react2.default.createElement('i', { className: 'fr-tooltip-icon' }),
        _react2.default.createElement(
          'div',
          { className: 'fr-tooltip-container' },
          _react2.default.createElement('i', { className: 'fr-tooltip-triangle' }),
          description
        )
      ) : _react2.default.createElement(
        'span',
        { className: 'fr-desc ml2' },
        '(\xA0',
        description,
        '\xA0)'
      )),
      displayType !== 'row' && showValidate && validateText && _react2.default.createElement(
        'span',
        { className: 'fr-validate' },
        validateText
      )
    ),
    _react2.default.createElement(
      'div',
      { className: contentClass },
      _react2.default.createElement(
        'div',
        { className: 'flex ' + (isComplex ? 'flex-column' : 'items-center') },
        children
      ),
      displayType === 'row' && showValidate && validateText && _react2.default.createElement(
        'span',
        { className: 'fr-validate absolute' },
        validateText
      )
    )
  );
};