'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.DefaultFieldUI = exports.asField = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _validate = require('./validate');

var _isHidden = require('./isHidden');

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

// asField拆分成逻辑组件和展示组件，从而可替换展示组件的方式完全插拔fr的样式
var asField = function asField(_ref) {
  var FieldUI = _ref.FieldUI,
      Widget = _ref.Widget;

  var FieldContainer = function FieldContainer(_ref2) {
    var className = _ref2.className,
        column = _ref2.column,
        showValidate = _ref2.showValidate,
        isRoot = _ref2.isRoot,
        hidden = _ref2.hidden,
        props = _ref2.props,
        showDescIcon = _ref2.showDescIcon,
        width = _ref2.width,
        labelWidth = _ref2.labelWidth,
        disabled = _ref2.disabled,
        readonly = _ref2.readonly,
        options = _ref2.options,
        schema = _ref2.schema,
        rest = _objectWithoutProperties(_ref2, ['className', 'column', 'showValidate', 'isRoot', 'hidden', 'props', 'showDescIcon', 'width', 'labelWidth', 'disabled', 'readonly', 'options', 'schema']);

    var displayType = rest.displayType,
        _rest$rootValue = rest.rootValue,
        rootValue = _rest$rootValue === undefined ? {} : _rest$rootValue,
        _rest$formData = rest.formData,
        formData = _rest$formData === undefined ? {} : _rest$formData,
        dependShow = rest.dependShow;
    // most key of schema, disabled, readonly, options, hidden, support for function expression

    var convertValue = function convertValue(item) {
      if (typeof item === 'function') {
        return item(formData, rootValue);
      } else if (typeof item === 'string' && (0, _utils.isFunction)(item) !== false) {
        var _item = (0, _utils.isFunction)(item);
        try {
          return (0, _utils.evaluateString)(_item, formData, rootValue);
        } catch (error) {
          console.error(error.message);
          console.error('happen at ' + item);
          return item;
        }
      }
      return item;
    };

    hidden = convertValue(hidden);
    disabled = convertValue(disabled);
    readonly = convertValue(readonly);
    options = convertValue(options);
    // iterate over schema, and convert every key
    var _schema = _extends({}, schema);
    Object.keys(schema).forEach(function (key) {
      var availableKey = ['title', 'description', 'format', 'pattern', 'message', 'min', 'max', 'step', 'enum', 'enumNames'];
      // TODO: need to cover more
      if (availableKey.indexOf(key) > -1) {
        _schema[key] = convertValue(schema[key]);
      }
    });

    // "ui:hidden": true, hide formItem
    // after "convertValue" being stable, this api will be discarded
    if (hidden && (0, _isHidden.isHidden)({ hidden: hidden, rootValue: rootValue, formData: formData })) {
      return null;
    }

    // 传入组件的值
    var _rest = _extends({}, rest, {
      schema: _schema,
      disabled: disabled,
      readonly: readonly,
      options: options,
      formData: formData || {},
      rootValue: rootValue || {}
    });

    // 不建议使用ui:dependShow, 一般一律使用ui:hidden。ui:dependShow可以做复杂、跨结构的校验
    if ((0, _isHidden.isDependShow)({ formData: formData, dependShow: dependShow })) {
      return null;
    }
    var isComplex = _schema.type === 'object' || _schema.type === 'array' && _schema.enum === undefined;

    var validateText = (0, _validate.getValidateText)(_rest);
    // 必填*，label，描述，竖排时的校验语，只要存在一个，label就不为空
    var showLabel = _schema.title || rest.description || rest.required || displayType !== 'row' && showValidate && validateText;

    var columnStyle = {};
    if (!isComplex && width) {
      columnStyle = {
        width: width,
        paddingRight: '24px'
      };
    } else if (!isComplex && column > 1) {
      columnStyle = {
        width: 'calc(100% /' + column + ')',
        paddingRight: '24px'
      };
    }

    var fieldProps = {
      className: className,
      columnStyle: columnStyle,
      displayType: displayType,
      isComplex: isComplex,
      isRequired: rest.required,
      isRoot: isRoot,
      schema: _schema,
      showDescIcon: showDescIcon,
      showLabel: showLabel,
      showValidate: showValidate,
      validateText: validateText,
      labelWidth: labelWidth
    };
    return _react2.default.createElement(
      FieldUI,
      fieldProps,
      _react2.default.createElement(Widget, _extends({}, _rest, { invalid: showValidate && validateText }))
    );
  };
  FieldContainer.propTypes = {
    showValidate: _propTypes2.default.bool,
    column: _propTypes2.default.number,
    isRoot: _propTypes2.default.bool,
    props: _propTypes2.default.object,
    showDescIcon: _propTypes2.default.bool,
    displayType: _propTypes2.default.string
  };

  FieldContainer.defaultProps = {
    showValidate: true,
    column: 1,
    isRoot: false,
    props: {},
    showDescIcon: false,
    displayType: 'column'
  };

  return FieldContainer;
};

exports.asField = asField;
var DefaultFieldUI = exports.DefaultFieldUI = function DefaultFieldUI(_ref3) {
  var children = _ref3.children,
      className = _ref3.className,
      columnStyle = _ref3.columnStyle,
      displayType = _ref3.displayType,
      isComplex = _ref3.isComplex,
      isRequired = _ref3.isRequired,
      isRoot = _ref3.isRoot,
      schema = _ref3.schema,
      showDescIcon = _ref3.showDescIcon,
      showLabel = _ref3.showLabel,
      showValidate = _ref3.showValidate,
      validateText = _ref3.validateText,
      labelWidth = _ref3.labelWidth;

  // field 整体 label 标签 content 内容
  var title = schema.title,
      type = schema.type,
      _enum = schema.enum,
      _schema$description = schema.description,
      description = _schema$description === undefined ? '' : _schema$description,
      widget = schema['ui:widget'];

  var isCheckbox = type === 'boolean' && widget !== 'switch';

  var fieldClass = 'fr-field w-100 ' + (isComplex ? 'fr-field-complex' : '');
  var labelClass = 'fr-label mb2';
  var contentClass = 'fr-content';

  switch (type) {
    case 'object':
      if (title) {
        labelClass += ' fr-label-object bb b--black-20 pb1 mt2 mb3'; // fr-label-object 无默认style，只是占位用于使用者样式覆盖
      }
      if (!isRoot) {
        fieldClass += ' fr-field-object'; // object的margin bottom由内部元素撑起
        if (title) {
          contentClass += ' ml3'; // 缩进
        }
      }
      break;
    case 'array':
      if (title && !_enum) {
        labelClass += ' fr-label-array mt2 mb3';
      }
      break;
    case 'boolean':
      if (isCheckbox) {
        if (title) {
          labelClass += ' ml2';
          labelClass = labelClass.replace('mb2', 'mb0');
        }
        contentClass += ' flex items-center'; // checkbox高度短，需要居中对齐
        fieldClass += ' flex items-center flex-row-reverse justify-end';
      }
      break;
    default:
      if (displayType === 'row') {
        labelClass = labelClass.replace('mb2', 'mb0');
      }
  }
  // 横排时
  if (displayType === 'row' && !isComplex && !isCheckbox) {
    fieldClass += ' flex items-center';
    labelClass += ' flex-shrink-0 fr-label-row';
    labelClass = labelClass.replace('mb2', 'mb0');
    contentClass += ' flex-grow-1 relative';
  }

  // 横排的checkbox
  if (displayType === 'row' && isCheckbox) {
    contentClass += ' flex justify-end pr2';
  }

  var _labelWidth = (0, _utils.isLooselyNumber)(labelWidth) ? Number(labelWidth) : (0, _utils.isCssLength)(labelWidth) ? labelWidth : 110; // 默认是 110px 的长度
  var labelStyle = { width: _labelWidth };
  if (isCheckbox) {
    labelStyle = { flexGrow: 1 };
  } else if (isComplex || displayType === 'column') {
    labelStyle = { flexGrow: 1 };
  }

  return _react2.default.createElement(
    'div',
    {
      className: className ? className + ' ' + fieldClass : fieldClass,
      style: columnStyle
    },
    showLabel && _react2.default.createElement(
      'div',
      { className: labelClass, style: labelStyle },
      _react2.default.createElement(
        'label',
        {
          className: 'fr-label-title ' + (isCheckbox || displayType === 'column' ? 'no-colon' : '') // boolean不带冒号
          , title: title
        },
        isRequired && _react2.default.createElement(
          'span',
          { className: 'fr-label-required' },
          ' *'
        ),
        _react2.default.createElement(
          'span',
          {
            className: (isComplex ? 'b' : '') + ' ' + (displayType === 'column' ? 'flex-none' : '')
          },
          title
        ),
        description && (showDescIcon ? _react2.default.createElement(
          'span',
          { className: 'fr-tooltip-toggle', 'aria-label': description },
          _react2.default.createElement('i', { className: 'fr-tooltip-icon' }),
          _react2.default.createElement(
            'div',
            { className: 'fr-tooltip-container' },
            _react2.default.createElement('i', { className: 'fr-tooltip-triangle' }),
            description
          )
        ) : _react2.default.createElement(
          'span',
          { className: 'fr-desc ml2' },
          '(\xA0',
          description,
          '\xA0)'
        )),
        displayType !== 'row' && showValidate && validateText && _react2.default.createElement(
          'span',
          { className: 'fr-validate' },
          validateText
        )
      )
    ),
    _react2.default.createElement(
      'div',
      {
        className: contentClass,
        style: isCheckbox ? displayType === 'row' ? { marginLeft: _labelWidth } : {} : { flexGrow: 1 }
      },
      _react2.default.createElement(
        'div',
        { className: 'flex ' + (isComplex ? 'flex-column' : 'items-center') },
        children
      ),
      displayType === 'row' && showValidate && validateText && _react2.default.createElement(
        'span',
        {
          className: 'fr-validate fr-validate-row ' + (isComplex ? 'relative' : 'absolute')
        },
        validateText
      )
    )
  );
};