'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _getField = require('./getField');

var _getField2 = _interopRequireDefault(_getField);

var _resolve = require('./resolve');

var _resolve2 = _interopRequireDefault(_resolve);

var _subFieldGenerator = require('./subFieldGenerator');

var _subFieldGenerator2 = _interopRequireDefault(_subFieldGenerator);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

// 对于数组或对象类型，获取其子集schema
function getSubSchemas() {
  var schema = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  var properties = schema.properties,
      items = schema.items,
      column = schema.column,
      $parent = _objectWithoutProperties(schema, ['properties', 'items', 'column']);

  var type = $parent.type;
  // no subset

  if (!properties && !items) {
    return [];
  }
  var children = {};
  if (type === 'object') {
    children = properties;
  }
  if (type === 'array') {
    children = [].concat(items);
  }
  return Object.keys(children).map(function (name) {
    return {
      schema: children[name],
      name: name,
      column: column,
      // parent propsSchema
      $parent: $parent
    };
  });
}

function getBasicProps(settings, materials) {
  var schema = settings.schema,
      _settings$name = settings.name,
      name = _settings$name === undefined ? '' : _settings$name,
      _settings$$parent = settings.$parent,
      $parent = _settings$$parent === undefined ? {} : _settings$$parent,
      column = settings.column,
      displayType = settings.displayType,
      showDescIcon = settings.showDescIcon,
      showValidate = settings.showValidate,
      readOnly = settings.readOnly,
      labelWidth = settings.labelWidth,
      useLogger = settings.useLogger,
      formData = settings.formData;
  // 写错的时候

  if (!schema) return {};

  // 目前做了处理的`uiSchema`参数
  var className = schema['ui:className'],
      _schema$uiOptions = schema['ui:options'],
      options = _schema$uiOptions === undefined ? {} : _schema$uiOptions,
      hidden = schema['ui:hidden'],
      disabled = schema['ui:disabled'],
      width = schema['ui:width'],
      readonly = schema['ui:readonly'],
      _schema$uiExtraButto = schema['ui:extraButtons'],
      extraButtons = _schema$uiExtraButto === undefined ? [] : _schema$uiExtraButto,
      dependShow = schema['ui:dependShow'],
      action = schema['ui:action'],
      _labelWidth = schema['ui:labelWidth'];
  var _$parent$required = $parent.required,
      required = _$parent$required === undefined ? [] : _$parent$required;
  var widgets = materials.generated,
      fields = materials.customized;
  // 标准化属性模型
  // 除了value和onChange为动态值这里不处理

  var basicProps = {
    name: name,
    schema: schema,
    column: column,
    displayType: displayType,
    showDescIcon: showDescIcon,
    showValidate: showValidate,
    options: options, // 所有特定组件规则，addable等规则TODO
    hidden: hidden,
    required: required.indexOf(name) !== -1,
    disabled: disabled,
    readonly: readOnly || readonly, // 前者全局的，后者单个ui的
    labelWidth: _labelWidth || labelWidth,
    useLogger: useLogger,
    width: width,
    widgets: widgets,
    fields: fields,
    formData: formData
  };
  // 假如有表达式来决定显示的场景，才传入dependShow,formData
  if (dependShow) {
    basicProps = _extends({}, basicProps, { dependShow: dependShow });
  }
  if (className) {
    basicProps = _extends({}, basicProps, { className: className });
  }
  if (action) {
    basicProps = _extends({}, basicProps, { action: action });
  }
  // 子集的属性
  var subItems = {};
  var subSchemas = getSubSchemas(schema);
  subSchemas.forEach(function (subSchema) {
    var _name = subSchema.name,
        _subSchema$schema = subSchema.schema,
        _schema = _subSchema$schema === undefined ? {} : _subSchema$schema;

    subItems[_name] = {
      field: (0, _getField2.default)(_schema, materials),
      props: getBasicProps(_extends({}, subSchema, {
        column: column,
        displayType: displayType,
        showDescIcon: showDescIcon,
        showValidate: showValidate,
        readOnly: readOnly,
        labelWidth: _labelWidth || labelWidth,
        useLogger: useLogger,
        formData: formData
      }), materials)
    };
  });
  if (['array', 'object'].indexOf(schema.type) >= 0) {
    // 传入name和Field（如果重定义Field的话）及其配置信息（如onChange等）
    basicProps.getSubField = function (o) {
      // getSchemaData(schema)
      var _ref = subItems[o.name] || subItems[0] || {},
          field = _ref.field,
          props = _ref.props,
          c = _ref.column;

      return (0, _subFieldGenerator2.default)(_extends({}, field, {
        column: c,
        props: _extends({}, props, {
          name: o.name,
          rootValue: o.rootValue
        })
      }))(o);
    };
    if (schema.type === 'array' && schema.items) {
      // 将数组uiSchema配置里面的抽离出来使用
      basicProps.extraButtons = extraButtons;
      // 数组新增的默认值
      if (subSchemas && subSchemas[0]) {
        basicProps.newItem = (0, _resolve2.default)(subSchemas[0].schema);
      }
    }
  }
  return basicProps;
}

/**
 *  schema + materials --> parse --> Field + props
 *  schema {
 *    propsSchema,
 *    uiSchema,
 *    data,
 *    name,
 *  }
 *  materials {
 *    // 根据 Widget 生成的 Field
 *    generated,
 *    // 自定义的 Field
 *    customized,
 *    // 字段 type 与 widgetName 的映射关系
 *    mapping,
 *  }
 */
var parse = function parse() {
  var settings = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var materials = arguments[1];

  console.log(settings);
  var _settings$schema = settings.schema,
      schema = _settings$schema === undefined ? {} : _settings$schema;

  return {
    Field: (0, _getField2.default)(schema, materials).Field,
    props: getBasicProps(settings, materials)
  };
};

exports.default = parse;