'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _utils = require('./utils');

// 获取当前字段默认值
function getDefaultValue(_ref) {
  var def = _ref.default,
      _ref$enum = _ref.enum,
      enums = _ref$enum === undefined ? [] : _ref$enum,
      type = _ref.type;

  var defaultValue = {
    array: [],
    boolean: false,
    integer: '',
    null: null,
    number: '',
    object: {},
    string: '',
    range: null
  };

  if ((0, _utils.isFunction)(def)) {
    return defaultValue[type];
  }
  if ((0, _utils.isFunction)(enums)) {
    if (type === 'array') {
      return [];
    }
    if (type === 'string' || type === 'number') {
      return '';
    }
  }

  // 如果设置默认值，优先从默认值中获取
  if (typeof def !== 'undefined') {
    return def;
  }
  // array且enum的情况，为多选框，默认值[]
  if (type === 'array' && enums.length) {
    return [];
  }
  // 如果enum是表达式，不处理
  // 如果设置枚举值，其次从枚举值中获取
  if (Array.isArray(enums) && enums[0] && typeof enums[0] !== 'undefined') {
    return enums[0];
  }
  // 最后使用对应基础类型的默认值
  return defaultValue[type];
}

function resolve(schema, data) {
  var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  var type = schema.type,
      properties = schema.properties,
      items = schema.items,
      def = schema.default,
      _schema$required = schema.required,
      required = _schema$required === undefined ? [] : _schema$required,
      widget = schema['ui:widget'];
  var _options$checkRequire = options.checkRequired,
      checkRequired = _options$checkRequire === undefined ? false : _options$checkRequire;
  // 当前值

  var value = typeof data === 'undefined' ? getDefaultValue(schema) : (0, _utils.clone)(data);
  if (type === 'object') {
    // 如果自定义组件
    if (widget) {
      if (def && (typeof def === 'undefined' ? 'undefined' : _typeof(def)) === 'object') {
        return def;
      }
      return value;
    }
    var subs = properties || {};
    var ret = {};
    Object.keys(subs).forEach(function (name) {
      var checkAndPass = checkRequired && [].concat(required).indexOf(name) !== -1;
      if (!checkRequired || checkAndPass) {
        ret[name] = resolve(subs[name], value[name], options);
      }
    });
    return ret;
  }
  if (type === 'array') {
    // 如果自定义组件
    if (widget) return value;
    if (def && Array.isArray(def) && !value) {
      return def;
    }
    var _subs = [].concat(items || []);
    var _ret = [];
    value.forEach && value.forEach(function (item, idx) {
      _ret[idx] = resolve(_subs[idx] || _subs[0], item, options);
    });
    return _ret;
  }
  return value;
}

exports.default = resolve;