'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getValidateList = exports.dealTypeValidate = exports.getValidateText = undefined;

var _isLength = require('validator/lib/isLength');

var _isLength2 = _interopRequireDefault(_isLength);

var _isHidden = require('./isHidden');

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } } /**
                                                                                                                                                                                                     * Created by Tw93 on 2018-09-04.
                                                                                                                                                                                                     * 校验表单格式
                                                                                                                                                                                                     */

var isEmptyObject = function isEmptyObject(obj) {
  return Object.keys(obj).length === 0 && obj.constructor === Object;
};

// 值是是否为空
var isEmptyValue = function isEmptyValue(value, schema) {
  // 多选组件的值为 [] 时，也判断为空值
  if (schema.type === 'array' && schema.enum) {
    return !value || value.length === 0;
  }
  if (value === 0) {
    return false;
  }
  return !value;
};

var getValidateText = exports.getValidateText = function getValidateText() {
  var obj = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var value = obj.value,
      defaultValue = obj.defaultValue,
      required = obj.required,
      _obj$schema = obj.schema,
      schema = _obj$schema === undefined ? {} : _obj$schema;
  var type = schema.type,
      pattern = schema.pattern,
      message = schema.message,
      format = schema.format,
      widget = schema['ui:widget'],
      minLength = schema.minLength,
      maxLength = schema.maxLength,
      minimum = schema.minimum,
      maximum = schema.maximum,
      minItems = schema.minItems,
      maxItems = schema.maxItems,
      uniqueItems = schema.uniqueItems;

  var finalValue = value || defaultValue;
  // fix: number = 0 返回空字符串
  if (type === 'number' && value === 0) {
    finalValue = 0;
  }
  var needPattern = pattern && ['string', 'number'].indexOf(type) > -1;
  // schema 里面没有内容的，直接退出
  if (isEmptyObject(schema)) {
    return false;
  }

  // 校验是否为required
  if (required && isEmptyValue(finalValue, schema)) {
    return message && message.required || '不能为空';
  }

  // 字符串相关校验
  if (type === 'string' && finalValue) {
    if (maxLength && !(0, _isLength2.default)(finalValue, 0, parseInt(maxLength, 10))) {
      return message && message.maxLength || '\u957F\u5EA6\u4E0D\u80FD\u5927\u4E8E ' + maxLength;
    }

    if ((minLength || minLength === 0) && !(0, _isLength2.default)(finalValue, parseInt(minLength, 10), undefined)) {
      return message && message.minLength || '\u957F\u5EA6\u4E0D\u80FD\u5C0F\u4E8E ' + minLength;
    }
    // 由于有人填写red，rgba(0,123,65,0.3)等值，所以暂时不做严格验证
    if (format === 'color' || widget === 'color') {
      var seemsLikeHex = finalValue.match(/^([0-9A-F]{6})$/i);
      if (seemsLikeHex) {
        return '颜色hex值请以#开头，例如#fff123';
      }
    }
  }

  // 数字相关校验
  if (type === 'number') {
    if (typeof finalValue !== 'number') {
      return '请填写数字';
    }
    if (maximum && parseInt(finalValue, 10) > maximum) {
      return message && message.maximum || '\u6570\u503C\u4E0D\u80FD\u5927\u4E8E ' + maximum;
    }
    if ((minimum || minimum === 0) && parseInt(finalValue, 10) < minimum) {
      return message && message.minimum || '\u6570\u503C\u4E0D\u80FD\u5C0F\u4E8E ' + minimum;
    }
  }

  // 正则只对数字和字符串有效果
  if (finalValue && needPattern && !new RegExp(pattern).test(finalValue)) {
    return message && message.pattern || '格式不匹配';
  }

  // 数组项目相关校验
  if (type === 'array') {
    if (maxItems && finalValue && finalValue.length > maxItems) {
      return message && message.maxItems || '\u7EC4\u6570\u4E0D\u80FD\u5927\u4E8E ' + maxItems;
    }

    if ((minItems || minItems === 0) && finalValue && finalValue.length < minItems) {
      return message && message.minItems || '\u7EC4\u6570\u4E0D\u80FD\u5C0F\u4E8E ' + minItems;
    }

    if (uniqueItems && Array.isArray(finalValue) && finalValue.length > 1) {
      if (typeof uniqueItems === 'boolean') {
        if ((0, _utils.hasRepeat)(finalValue)) {
          return '存在重复元素';
        }
      }
      if (typeof uniqueItems === 'string') {
        var nameList = finalValue.map(function (item) {
          return item[uniqueItems];
        });
        var isRepeat = nameList.find(function (x, index) {
          return nameList.indexOf(x) !== index;
        });
        if (isRepeat) {
          return '存在重复的 ' + uniqueItems + ' 值';
        }
      }
    }
  }

  return false;
};

var dealTypeValidate = exports.dealTypeValidate = function dealTypeValidate(key, value) {
  var schema = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

  var checkList = [];
  var type = schema.type,
      items = schema.items;

  var obj = {
    value: value,
    schema: schema
  };
  if (type === 'object') {
    var list = getValidateList(value, schema); // eslint-disable-line
    checkList.push.apply(checkList, _toConsumableArray(list));
  } else if (type === 'array') {
    value.forEach(function (v) {
      var list = dealTypeValidate(key, v, items);
      checkList.push.apply(checkList, _toConsumableArray(list));
    });
  }
  if (getValidateText(obj)) {
    checkList.push(key);
  }
  return checkList;
};

// for backward compatibility
var keyHidden = function keyHidden(schema, val) {
  var hidden = schema && schema['ui:hidden'];
  if (typeof hidden === 'string' && (0, _utils.isFunction)(hidden) === false) {
    hidden = (0, _isHidden.isHidden)({ hidden: hidden, rootValue: val });
  }
  return hidden;
};

var getValidateList = exports.getValidateList = function getValidateList() {
  var val = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var prop = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  var checkList = [];
  var properties = prop.properties,
      required = prop.required;
  // 校验必填（required 属性只在 type:object 下存在）

  if (required && required.length > 0) {
    required.forEach(function (key) {
      var schema = properties && properties[key] || {};
      var hidden = keyHidden(schema, val);
      var itemValue = val && val[key];
      if (isEmptyValue(itemValue, schema) && !hidden) {
        checkList.push(key);
      }
    });
  }

  if (properties && val && Object.keys(val) && Object.keys(val).length > 0) {
    Object.keys(val).forEach(function (key) {
      var value = val[key];
      var schema = properties[key] || {};
      var hidden = keyHidden(schema, val);
      if (!hidden) {
        var list = dealTypeValidate(key, value, schema);
        checkList.push.apply(checkList, _toConsumableArray(list));
      }
    });
  }

  return checkList;
};