'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getDescription = undefined;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _isHidden = require('../base/isHidden');

var _utils = require('../base/utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var getEnumValue = function getEnumValue(value, enums, enumNames) {
  if (Array.isArray(enums) && Array.isArray(enumNames)) {
    if (typeof value === 'string' || typeof value === 'number') {
      var count = enums.indexOf(value);
      if (count > -1) {
        return enumNames[count];
      }
      return value;
    } else if (Array.isArray(value)) {
      var result = value.map(function (v) {
        return getEnumValue(value, enums, enumNames);
      });
      return String(result);
    }
    return value;
  }
  return value;
};

var DescriptionList = function DescriptionList(_ref) {
  var _ref$schema = _ref.schema,
      schema = _ref$schema === undefined ? {} : _ref$schema,
      _ref$value = _ref.value,
      value = _ref$value === undefined ? [] : _ref$value,
      index = _ref.index;

  var list = getDescription({ schema: schema, value: value, index: index }).filter(function (item) {
    return item.title;
  }).slice(0, 3);
  return _react2.default.createElement(
    'ul',
    { className: 'flex overflow-hidden', style: { paddingRight: 45 } },
    list.map(function (item, i) {
      return item.title ? _react2.default.createElement(
        'li',
        {
          className: 'overflow-hidden truncate',
          style: { width: '33%', paddingRight: 8 },
          key: i
        },
        _react2.default.createElement(
          'span',
          { className: 'fw5' },
          item.title,
          ': '
        ),
        _react2.default.createElement(
          'span',
          { className: 'truncate' },
          item.text
        )
      ) : null;
    })
  );
};

exports.default = DescriptionList;

// 获得title，value值list

var getDescription = exports.getDescription = function getDescription(_ref2) {
  var _ref2$schema = _ref2.schema,
      schema = _ref2$schema === undefined ? {} : _ref2$schema,
      _ref2$value = _ref2.value,
      value = _ref2$value === undefined ? [] : _ref2$value,
      index = _ref2.index;
  var _schema$items = schema.items,
      items = _schema$items === undefined ? {} : _schema$items;
  // 只有当items为object时才做收起（fold）处理

  if (items.type !== 'object') {
    return [];
  }
  var titles = items && items.properties || {};
  titles = Object.values(titles);
  var description = value && value.length && value[index] || {};
  var valueList = Object.values(description);
  var descList = titles.map(function (t, idx) {
    var hidden = t && t['ui:hidden'];
    // ui:hidden为判断式时解析 TODO: 解析在外部集中做
    if (typeof hidden === 'string' && (0, _utils.isFunction)(hidden) === false) {
      hidden = (0, _isHidden.isHidden)({ hidden: hidden, rootValue: description });
    }
    if (hidden) return;
    var title = t.title;
    var text = valueList[idx];
    if (text === null && text === undefined) {
      text = '';
    } else if (typeof text === 'boolean') {
      text = text ? '是' : '否';
    } else if (typeof text !== 'string' && typeof text !== 'number' && text) {
      text = '{复杂结构}';
    } else if (t.enum && t.enumNames) {
      text = getEnumValue(text, t.enum, t.enumNames);
    }
    return {
      title: title,
      text: text
    };
  });
  // 去空
  return descList.filter(function (d) {
    return !!d;
  });
};