'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _hooks = require('./hooks');

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _utils = require('./base/utils');

var _asField = require('./base/asField');

var _parser = require('./base/parser');

var _parser2 = _interopRequireDefault(_parser);

var _resolve = require('./base/resolve');

var _resolve2 = _interopRequireDefault(_resolve);

var _validate = require('./base/validate');

require('../atom.css');

require('../index.css');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function renderField(settings, fields, events) {
  var _parse = (0, _parser2.default)(settings, fields),
      Field = _parse.Field,
      props = _parse.props;

  if (!Field) {
    return null;
  }
  return _react2.default.createElement(Field, _extends({
    isRoot: true
  }, props, {
    value: settings.data
  }, events, {
    formData: settings.formData
  }));
}

// 在顶层将 propsSchema 和 uiSchema 合并，便于后续处理。 也可直接传入合并的 schema
var Wrapper = function Wrapper(_ref) {
  var schema = _ref.schema,
      _ref$propsSchema = _ref.propsSchema,
      propsSchema = _ref$propsSchema === undefined ? {} : _ref$propsSchema,
      _ref$uiSchema = _ref.uiSchema,
      uiSchema = _ref$uiSchema === undefined ? {} : _ref$uiSchema,
      readOnly = _ref.readOnly,
      showValidate = _ref.showValidate,
      rest = _objectWithoutProperties(_ref, ['schema', 'propsSchema', 'uiSchema', 'readOnly', 'showValidate']);

  var _schema = schema ? schema : (0, _utils.combineSchema)(propsSchema, uiSchema);

  return _react2.default.createElement(FormRender, _extends({
    readOnly: readOnly,
    showValidate: !readOnly && showValidate // 预览模式下不展示校验
  }, rest, {
    schema: _schema
  }));
};

function FormRender(_ref2) {
  var _ref2$name = _ref2.name,
      name = _ref2$name === undefined ? '$form' : _ref2$name,
      _ref2$column = _ref2.column,
      column = _ref2$column === undefined ? 1 : _ref2$column,
      className = _ref2.className,
      _ref2$schema = _ref2.schema,
      schema = _ref2$schema === undefined ? {} : _ref2$schema,
      _ref2$formData = _ref2.formData,
      formData = _ref2$formData === undefined ? {} : _ref2$formData,
      _ref2$widgets = _ref2.widgets,
      widgets = _ref2$widgets === undefined ? {} : _ref2$widgets,
      _ref2$FieldUI = _ref2.FieldUI,
      FieldUI = _ref2$FieldUI === undefined ? _asField.DefaultFieldUI : _ref2$FieldUI,
      _ref2$fields = _ref2.fields,
      fields = _ref2$fields === undefined ? {} : _ref2$fields,
      _ref2$mapping = _ref2.mapping,
      mapping = _ref2$mapping === undefined ? {} : _ref2$mapping,
      _ref2$showDescIcon = _ref2.showDescIcon,
      showDescIcon = _ref2$showDescIcon === undefined ? false : _ref2$showDescIcon,
      _ref2$showValidate = _ref2.showValidate,
      showValidate = _ref2$showValidate === undefined ? true : _ref2$showValidate,
      _ref2$displayType = _ref2.displayType,
      displayType = _ref2$displayType === undefined ? 'column' : _ref2$displayType,
      _ref2$onChange = _ref2.onChange,
      _onChange = _ref2$onChange === undefined ? function () {} : _ref2$onChange,
      _ref2$onValidate = _ref2.onValidate,
      onValidate = _ref2$onValidate === undefined ? function () {} : _ref2$onValidate,
      onMount = _ref2.onMount,
      _ref2$readOnly = _ref2.readOnly,
      readOnly = _ref2$readOnly === undefined ? false : _ref2$readOnly,
      _ref2$labelWidth = _ref2.labelWidth,
      labelWidth = _ref2$labelWidth === undefined ? 110 : _ref2$labelWidth,
      _ref2$useLogger = _ref2.useLogger,
      useLogger = _ref2$useLogger === undefined ? false : _ref2$useLogger;

  var originWidgets = (0, _react.useRef)();
  var generatedFields = (0, _react.useRef)({});
  var previousSchema = (0, _hooks.usePrevious)(schema);

  var data = (0, _react.useMemo)(function () {
    return (0, _resolve2.default)(schema, formData);
  }, [schema, formData]);

  (0, _react.useEffect)(function () {
    if (onMount) {
      onMount(data);
    } else {
      _onChange(data);
    }
    onValidate((0, _validate.getValidateList)(data, schema));
  }, []);

  (0, _react.useEffect)(function () {
    if (!(0, _utils.isDeepEqual)(previousSchema, schema)) {
      needUpdateForm();
    }
  }, [schema]);

  // 如果 onMount props存在且入参useOnMount = true，使用onMount代替onChange，用于首次加载的特别逻辑
  var needUpdateForm = function needUpdateForm() {
    _onChange(data);
    onValidate((0, _validate.getValidateList)(data, schema));
  };

  var generated = {};
  if (!originWidgets.current) {
    originWidgets.current = widgets;
  }
  Object.keys(widgets).forEach(function (key) {
    var oWidget = originWidgets.current[key];
    var nWidget = widgets[key];
    var gField = generatedFields.current[key];
    if (!gField || oWidget !== nWidget) {
      if (oWidget !== nWidget) {
        originWidgets.current[key] = nWidget;
      }
      gField = (0, _asField.asField)({ FieldUI: FieldUI, Widget: nWidget });
      generatedFields.current[key] = gField;
    }
    generated[key] = gField;
  });

  return _react2.default.createElement(
    'div',
    { className: className },
    renderField({
      schema: schema,
      data: data,
      name: name,
      column: column,
      showDescIcon: showDescIcon,
      showValidate: showValidate,
      displayType: displayType,
      readOnly: readOnly,
      labelWidth: labelWidth,
      useLogger: useLogger,
      formData: data
    }, {
      // 根据 Widget 生成的 Field
      generated: generated,
      // 自定义的 Field
      customized: fields,
      // 字段 type 与 widgetName 的映射关系
      mapping: mapping
    }, {
      onChange: function onChange(key, val) {
        _onChange(val);
        onValidate((0, _validate.getValidateList)(val, schema));
      }
    })
  );
}

FormRender.propTypes = {
  name: _propTypes2.default.string,
  column: _propTypes2.default.number,
  schema: _propTypes2.default.object,
  formData: _propTypes2.default.object,
  widgets: _propTypes2.default.objectOf(_propTypes2.default.func),
  FieldUI: _propTypes2.default.elementType,
  fields: _propTypes2.default.objectOf(_propTypes2.default.element),
  mapping: _propTypes2.default.object,
  showDescIcon: _propTypes2.default.bool,
  showValidate: _propTypes2.default.bool,
  displayType: _propTypes2.default.string,
  onChange: _propTypes2.default.func,
  onMount: _propTypes2.default.func,
  onValidate: _propTypes2.default.func,
  readOnly: _propTypes2.default.bool,
  labelWidth: _propTypes2.default.number,
  useLogger: _propTypes2.default.bool
};

exports.default = Wrapper;